import {Component} from '@angular/core';
import {BackendService, ScrumSprint} from '../../../services/backend.service';
import {TableComponentBase} from '../table-component.base';
import {NgbModal} from '@ng-bootstrap/ng-bootstrap';
import {ActivatedRoute, ParamMap, Router} from '@angular/router';
import { SprintFormComponent } from '../../sprint-form/sprint-form.component';

@Component({
  selector: 'app-sprint',
  templateUrl: './sprint-table.component.html',
  styleUrls: ['./sprint-table.component.css']
})
export class SprintTableComponent extends TableComponentBase<ScrumSprint> {
  constructor(
    private backendService: BackendService, private modalService: NgbModal,
    protected route: ActivatedRoute, private router: Router
  ) {
    super(route);

    backendService.getSprints().subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.items.push(...response.body);
      }
    });
  }

  //#region getters
  //#endregion getters

  //#region sprintTableFunctions
  public deleteSprint(sprint: ScrumSprint) {
    this.backendService.deleteSprint(sprint).subscribe(response => {
      if (response.status > 399) {
        alert('Fehler');
      }
    });
    const index = this.items.indexOf(sprint);
    if (index !== -1) {
      this.items.splice(index, 1);
    }
  }
  //#endregion sprintTableFunctions

  //#region sorters
  sortByStartDate() {
    this.doStringSort('startDate', sprint => sprint.startDate);
  }

  sortByEndDate() {
    this.doStringSort('endDate', sprint => sprint.endDate);
  }
  //#endregion sorters

  //#region modals
  public openSprintForm(editSprint?: ScrumSprint) {
    const modalRef = this.modalService.open(SprintFormComponent, {
      backdrop: 'static',
      keyboard: true,
      size: "md",
    });
    if (editSprint === null) {
      modalRef.result.then(result => {
        this.items.push(result);
      });
    }
    modalRef.componentInstance.sprint = editSprint;
  }
  //#endregion modals
}
